
local MAJOR = "StarLibLayoutOptions-1.0" 
local MINOR = 11
assert(LibStub, MAJOR.." requires LibStub") 
local LayoutOptions = LibStub:NewLibrary(MAJOR, MINOR)
if not LayoutOptions then return end


function LayoutOptions:RebuildOpts(visitor, db, name) 
	local options = {
		name = {
			name = name,
			type = "input",
			get = function() return db.name end,
			set = function(info, v) db.name = v end,
			order = 1
		},
		rows = {
			name = "Rows",
			type  = "group",
			args = {
				addRow = {
					name = "Add Row",
					desc = "Add a row",
					type = "input",
					pattern = "%d",
					set = function(info, v)
						db["row" .. v] = {}
						visitor:RebuildOpts()
					end,
					order = 1
				}
			}
		}
	}
	
	for row, v in pairs(db) do
		if type(row) ~= "number" then break end
		options.rows.args["row" ..row] = {
			name = "Row " .. row,
			type = "group",
			args = {
				add = {
					name = "Add Column",
					desc = "Add a column",
					type = "input",
					pattern = "%d",
					set = function(info, v)
						db[row][v] = "<placeholder>"
					end,
					order = 1
				},
			},
			order = row + 1
		}
		for col, vv in pairs(v) do
			if type(col) ~= "number" then break end
			options.rows.args["row" .. row].args["col" .. col] = {
				name = "Column" .. col,
				desc = "Input a widget name",
				type = "input",
				get = function() return vv end,
				set = function(info, val) 
					db[row][col] = val 
					visitor:RebuildOpts()
				end,
				order = col + 1
			}
		end
	end
	
	return options
end
